// Drive the ESCs
#include "Arduino.h"
#include "Flux.hpp"

extern volatile int rcThrottle; // For safety and tuning functions only

extern  int throttleControl();
extern  int pitchControl();
extern  int rollControl();
extern  int yawControl();

void controlEscs(Tune tune, unsigned long loopStartTime) {

  PORTD |= B11110000; // Start ESC control pulses y generating rising edges: Set digital outputs 4,5,6,7 to high.

  // Calculate ESC output ------------------------------
  int pwThrottle = throttleControl();
  int pwPitch = pitchControl();
  int pwRoll = rollControl();
  int pwYaw = yawControl();

  // Target range for esc pulse widths is 1000 - 2000
  int pwFR = pwThrottle + pwPitch - pwRoll + pwYaw;
  int pwRR = pwThrottle - pwPitch - pwRoll - pwYaw;
  int pwRL = pwThrottle - pwPitch + pwRoll + pwYaw;
  int pwFL = pwThrottle + pwPitch + pwRoll - pwYaw;

  if (tune == Tune::OFF) {
    if (rcThrottle < THROTTLE_SAFE) {
      // avoid propellers rotating when quadcopter is picked up manually and held inclined
      pwFR = rcThrottle;
      pwRR = rcThrottle;
      pwRL = rcThrottle;
      pwFL = rcThrottle;
    } else {
      // regular operation
      pwFR = constrain(pwFR, 1000, 2000);
      pwRR = constrain(pwRR, 1000, 2000);
      pwRL = constrain(pwRL, 1000, 2000);
      pwFL = constrain(pwFL, 1000, 2000);
    }
  } else {
    // tuning functions
    pwFR = pwRR = pwRL = pwFL = 1000;
    switch (tune) {
      case Tune::CALIBRATE_ESCS:
        pwFR = pwRR = pwRL = pwFL = rcThrottle;
        break;
      case Tune::BALANCE_PROP_FR:
        pwFR = rcThrottle;
        break;
      case Tune::BALANCE_PROP_RR:
        pwRR = rcThrottle;
        break;
      case Tune::BALANCE_PROP_RL:
        pwRL = rcThrottle;
        break;
      case Tune::BALANCE_PROP_FL:
        pwFL = rcThrottle;
        break;
      default:
        break;
    }
  }

  // Generate falling edges of ESC control pulses -----
  byte escSet = 0b1111;
  while (escSet) {
    int now = micros() - loopStartTime;
    if (now > pwFR) {
      PORTD &= B11101111;  // Set digital output 4 to low if the time has expired.
      escSet &= 0b1110;    // Clear esc 1 flag
    }
    if (now > pwRR) {
      PORTD &= B11011111;  // Set digital output 5 to low if the time has expired.
      escSet &= 0b1101;    // Clear esc 2 flag
    }
    if (now > pwRL) {
      PORTD &= B10111111;  // Set digital output 6 to low if the time has expired.
      escSet &= 0b1011;    // Clear esc 3 flag
    }
    if (now > pwFL) {
      PORTD &= B01111111;  // Set digital output 7 to low if the time has expired.
      escSet &= 0b0111;    // Clear esc 4 flag
    }
  }
}
