// Interrupt service routine measuring incoming remote control pulse widths
#include "Arduino.h"

// Output values:  (values in range 1000..2000)
volatile int rcThrottle, rcRoll, rcPitch, rcYaw;

// Local variables:
static bool highPin8, highPin9, highPin10, highPin11;
static unsigned long timerPin8, timerPin9, timerPin10, timerPin11;

ISR(PCINT0_vect) {
  unsigned long now = micros();
  // Pin 8 ----------------------------------------------------------
  if (PINB & B00000001) {            // Is pin 8 high?
    if (!highPin8) {                 // Was it low before?
      highPin8 = true;               // If yes, store high state
      timerPin8 = now;               // start timer
    }
  } else if (highPin8) {             // Was it high before?
    highPin8 = false;                // If yes, store low state
    rcRoll = now - timerPin8;        // Output measured value
  }
  // Pin 9 ----------------------------------------------------------
  if (PINB & B00000010) {            // Is pin 9 high?
    if (!highPin9) {                 // Was it low before?
      highPin9 = true;               // If yes, store high state
      timerPin9 = now;               // start timer
    }
  } else if (highPin9) {             // Was it high before?
    highPin9 = false;                // If yes, store low state
    rcPitch = now - timerPin9;       // Output measured value
  }
  // Pin 10 ----------------------------------------------------------
  if (PINB & B00000100) {            // Is pin 10 high?
    if (!highPin10) {                // Was it low before?
      highPin10 = true;              // If yes, store high state
      timerPin10 = now;              // start timer
    }
  } else if (highPin10) {            // Was it high before?
    highPin10 = false;               // If yes, store low state
    rcThrottle = now - timerPin10;   // Output measured value
  }
  // Pin 11 ----------------------------------------------------------
  if (PINB & B00001000) {            // Is pin 11 high?
    if (!highPin11) {                // Was it low before?
      highPin11 = true;              // If yes, store high state
      timerPin11 = now;              // start timer
    }
  } else if (highPin11) {            // Was it high before?
    highPin11 = false;               // If yes, store low state
    rcYaw = now - timerPin11;        // Output measured value
  }
}
