#include "Arduino.h"
#include<Wire.h>
#include "Flux.hpp"

int accX, accY, accZ, gyroX, gyroY, gyroZ;
float temperature;

const int IMU_addr = 0x68;      // I2C address of the MPU-6050
const int PWR_MGMT_1 = 0x6B;    // register address
const int CONFIG = 0x1A;        // register address
const int GYRO_CONFIG = 0x1B;   // register address
const int ACCEL_CONFIG = 0x1C;  // register address

void setupImu() {
  Wire.begin();
  TWBR = 12;                // Set the I2C clock speed to 400kHz.

  Wire.beginTransmission(IMU_addr);
  Wire.write(PWR_MGMT_1);
  Wire.write(0);            // wake up the MPU-6050
  Wire.endTransmission();

  Wire.beginTransmission(IMU_addr);
  Wire.write(GYRO_CONFIG);
  Wire.write(0x08);         // select sensitivity (+/- 500 degree/s)
  Wire.endTransmission();

  Wire.beginTransmission(IMU_addr);
  Wire.write(ACCEL_CONFIG);
  Wire.write(0x10);         // select sensitivity (+/- 8g)
  Wire.endTransmission();

  Wire.beginTransmission(IMU_addr);
  Wire.write(CONFIG);
  Wire.write(0x03);         // Set Digital Low Pass Filter to ~43Hz
  Wire.endTransmission();

  delay(1000);              // Warm-up
}

void readImu() {
  const int ACCEL_XOUT_H = 0x3B;
  Wire.beginTransmission(IMU_addr);
  Wire.write(ACCEL_XOUT_H);   //Start reading register ACCEL_XOUT_H and auto increment with every read.
  Wire.endTransmission();

  Wire.requestFrom(IMU_addr, 14);  //Request 14 bytes from the gyro.
  while (Wire.available() < 14) ;  //Wait until all bytes are received.
  accX = Wire.read() << 8 | Wire.read();        // ACCEL_XOUT_H, ACCEL_XOUT_L
  accY = Wire.read() << 8 | Wire.read();        // ACCEL_YOUT_H, ACCEL_YOUT_L
  accZ = Wire.read() << 8 | Wire.read();        // ACCEL_ZOUT_H, ACCEL_ZOUT_L
  int temp = Wire.read() << 8 | Wire.read();    // TEMP_OUT_H, TEMP_OUT_L
  gyroX = Wire.read() << 8 | Wire.read();       // GYRO_XOUT_H, GYRO_XOUT_L
  gyroY = Wire.read() << 8 | Wire.read();       // GYRO_YOUT_H, GYRO_YOUT_L
  gyroZ = Wire.read() << 8 | Wire.read();       // GYRO_ZOUT_H, GYRO_ZOUT_L
  temperature = temp / 340.0 + 36.53;
}
