#include "Arduino.h"
#include "eeprom.h"
#include "Flux.hpp"

extern FluxEeprom eeprom;
extern void remoteControlService();
extern void imuTest();
extern void horizontalCalibration();
extern void motorTest();
extern void pidGainAdjust();
extern Tune calibrateESCs();
extern Tune balanceProps();
extern void readEeprom();
extern int calculateEepromChecksum();
extern bool isEepromChecksumCorrect();
extern bool writeEeprom();

// Not sure why these forward declarations are needed
Tune menu();
void dumpEeprom();

Tune service() {
  Serial.begin(57600);
  readEeprom();
  if (isEepromChecksumCorrect()) {
    Serial.println(F("Pruefsumme im EEPROM ist korrekt."));
  } else {
    Serial.println(F("ACHTUNG: Pruefsumme in EEPROM ist nicht korrekt"));
    Serial.print(F("Berechnet: ")); Serial.println(calculateEepromChecksum(), HEX);
    Serial.print(F("Tatsaechlich: ")); Serial.println(eeprom.checksum, HEX);
  }
  Serial.flush();

  Tune tune = Tune::OFF;
  do {
    tune = menu();
  } while (tune == Tune::OFF);

  return tune;
}

Tune menu()  {
  Serial.println();
  Serial.println(F("=========== FLUX Dialog Wartung ============"));
  // Print menu
  Serial.println(F("Bitte Zeichen zur Funktionsauswahl eingeben:"));
  Serial.println(F("1 - Fernsteuerung Test und Kalibration"));
  Serial.println(F("2 - IMU Test"));
  Serial.println(F("3 - Horizontal-Kalibrierung"));
  Serial.println(F("4 - PID-Einstellung Lageregelung"));
  Serial.println(F("5 - ESC-Kalibration"));
  Serial.println(F("6 - Propeller-Balancierung"));
  Serial.println(F("a - EEPROM-Puffer anzeigen"));
  Serial.println(F("s - EEPROM schreiben"));
  Serial.println();
  Serial.flush();
  while (Serial.available() == 0) ; // Wait for input

  byte data = Serial.read();
  (void) Serial.read(); // Consume newline
  Tune tune = Tune::OFF;
  switch (data) {
    case '1':  remoteControlService();   break;
    case '2':  imuTest();                break;
    case '3':  horizontalCalibration();  break;
    case '4':  pidGainAdjust();          break;
    case '5':  tune = calibrateESCs();   break;
    case '6':  tune = balanceProps();    break;
    case 'a':  dumpEeprom();             break;
    case 's':  writeEeprom();            break;
    default:
      Serial.println(F("Auswahl nicht verstanden - bitte wiederholen!"));
      return tune;
  }
  return tune;
}

void dumpEeprom() {
  Serial.println(F("Inhalt des EEPROM-Puffers:"));
  Serial.println(F("--------------------------"));
  Serial.print(F("Signatur = "));
  char* pChar = (char*) &eeprom.signature;
  for (int i = 0; i < 4; i++) {
    Serial.print(*pChar++);
  }
  Serial.print(F(", Revision "));
  Serial.println(eeprom.revision);
  Serial.println(F("-- Accelerometer zero values (values while in horizontal position)"));
  Serial.print(F("accXhorizontal = ")); Serial.println(eeprom.accXhorizontal);
  Serial.print(F("accYhorizontal = ")); Serial.println(eeprom.accYhorizontal);
  Serial.print(F("accZhorizontal = ")); Serial.println(eeprom.accZhorizontal);
  Serial.println(F("-- PID values for leveling control"));
  Serial.print(F("pPitchRoll = ")); Serial.println(eeprom.pPitchRoll);
  Serial.print(F("iPitchRoll = ")); Serial.println(eeprom.iPitchRoll);
  Serial.print(F("dPitchRoll = ")); Serial.println(eeprom.dPitchRoll);
  Serial.println(F("-- PID values for yaw control"));
  Serial.print(F("pYaw = ")); Serial.println(eeprom.pYaw);
  Serial.print(F("iYaw = ")); Serial.println(eeprom.iYaw);
  Serial.print(F("dYaw = ")); Serial.println(eeprom.dYaw);
  Serial.println(F("-- Checksum"));
  Serial.print(F("checksum = ")); Serial.println(eeprom.checksum, HEX);
}

