#include "Arduino.h"
#include <EEPROM.h>
#include "eeprom.h"

FluxEeprom eeprom;

void readEeprom() {
  byte* ptr = (byte*) &eeprom;
  for (unsigned int address = 0; address < sizeof(FluxEeprom); address++) {
    *ptr++ = EEPROM.read(address);
  }
}

int calculateEepromChecksum() {  // After https://en.wikipedia.org/wiki/Fletcher's_checksum
  int sum1 = 0;
  int sum2 = 0;
  byte* data = (byte*) &eeprom;
  int bytecount = sizeof(FluxEeprom) - 2;
  for (int index = 0; index < bytecount; ++index)
  {
    sum1 = (sum1 + data[index]) % 255;
    sum2 = (sum2 + sum1) % 255;
  }
  return (sum2 << 8) | sum1;
}

bool isEepromChecksumCorrect() {
  return eeprom.checksum == calculateEepromChecksum();
}

bool writeEeprom() {
  char* sig = (char *) &eeprom.signature;
  sig[0] = 'F';
  sig[1] = 'L';
  sig[2] = 'U';
  sig[3] = 'X';
  eeprom.revision = 1;
  eeprom.checksum = calculateEepromChecksum();
  // Copy buffer to EEPROM
  byte* ptr = (byte*) &eeprom;
  for (unsigned int address = 0; address < sizeof(FluxEeprom); address++) {
    EEPROM.write(address, *ptr++);
  }
  // Verify result
  ptr = (byte*) &eeprom;
  for (unsigned int address = 0; address < sizeof(FluxEeprom); address++, ptr++) {
    if (*ptr != EEPROM.read(address)) {
      Serial.print(F("Fehler beim Beschreiben des EEPROM an der Adresse "));
      Serial.print(address);
      Serial.print(F(". Erwarteter Inhalt: "));
      Serial.print("*ptr");
      Serial.print(F(". Tatsächlicher Inhalt: "));
      Serial.print(EEPROM.read(address));
      return false;
    }
  }
  Serial.print(F("Das EEPROM wurde erfolgreich beschrieben, Pruefsumme = "));
  Serial.println(eeprom.checksum, HEX);
  return true;
}

